@@ -5,7 +5,7 @@
 @(#)Purpose:         Specialized doubly-linked list management routines
 @(#)Author:          J Leffler
 @(#)Copyright:       (C) Jonathan Leffler 1996,1997
-@(#)Product:         :PRODUCT:
+@(#)Product:         $Product: DBD::Informix Version 0.57 (1997-11-13) $
 */
 
 /*TABSTOP=4*/
